--
-- $Id: test_bench.vhdl,v 1.5 2009-10-15 16:03:27 vrsieh Exp $
--
-- Copyright (C) 2007-2009 FAUmachine Team <info@faumachine.org>.
--
-- This program is free software. You can redistribute it and/or modify it
-- under the terms of the GNU General Public License, either version 2 of
-- the License, or (at your option) any later version. See COPYING.
--

library expect;
use expect.types.all;
use expect.procedures.all;
library ieee;
use ieee.std_logic_1164.ALL;

entity test_bench is
end test_bench;

architecture structural of test_bench is
	signal ext_eth0 : eth;
	signal ext_usb0 : usb_bus;
	signal ext_usb1 : usb_bus;
	signal ext_par : parallel;
	signal ext_ser0 : serial;
	signal ext_ser1 : serial;
	signal ext_ser2 : serial;

	signal power_switch : boolean;
	signal power_button : boolean;
	signal reset_button : boolean;

	signal media_floppy : floppy;
	signal floppy : cstring;
	signal media_cd : magneto_optical;
	signal cdrom : cstring;
	signal floppy_conn : floppy_array(1 to 20);
	signal cddvd_conn : magneto_optical_array(1 to 20);

	signal key : boolean_array(0 to 127);

	signal mouse_dx : integer;
	signal mouse_dy : integer;
	signal mouse_dz : integer;
	signal mouse_button1 : boolean;
	signal mouse_button2 : boolean;
	signal mouse_button3 : boolean;
	signal mouse_button4 : boolean;
	signal mouse_button5 : boolean;

	signal screen_shot : boolean;
	signal rec : boolean;

	signal term_power_switch : boolean;

	signal term_key : boolean_array(0 to 127);

	signal term_screen_shot : boolean;
	signal term_rec : boolean;

	signal opt_screen : opt_rgb;
	signal opt_vt102 : opt_rgb;
begin
	pc : pc_standard
		port map(
			ext_eth0 => ext_eth0,
			ext_usb0 => ext_usb0,
			ext_usb1 => ext_usb1,
			ext_par => ext_par,
			ext_ser0 => ext_ser0,
			ext_ser1 => ext_ser1,
			power_switch => power_switch,
			power_button => power_button,
			reset_button => reset_button,
			media_floppy => media_floppy,
			floppy => floppy,
			media_cd => media_cd,
			cdrom => cdrom,
			key => key,
			mouse_dx => mouse_dx,
			mouse_dy => mouse_dy,
			mouse_dz => mouse_dz,
			mouse_button1 => mouse_button1,
			mouse_button2 => mouse_button2,
			mouse_button3 => mouse_button3,
			mouse_button4 => mouse_button4,
			mouse_button5 => mouse_button5,
			screen_shot => screen_shot,
			rec => rec,
			opt_screen => opt_screen
		);
	usb2serial : usb2serial
		port map(
			serial => ext_ser2,
			usb => ext_usb0
		);
	vt102 : vt102
		port map(
			power_switch => term_power_switch,
			serial => ext_ser2,
			key => term_key,
			screen_shot => term_screen_shot,
			rec => term_rec,
			opt_video => opt_vt102
		);
	user : user
		port map(
			power_switch => power_switch,
			power_button => power_button,
			reset_button => reset_button,
			floppy => floppy,
			cdrom => cdrom,
			key => key,
			mouse_dx => mouse_dx,
			mouse_dy => mouse_dy,
			mouse_dz => mouse_dz,
			mouse_button1 => mouse_button1,
			mouse_button2 => mouse_button2,
			mouse_button3 => mouse_button3,
			mouse_button4 => mouse_button4,
			mouse_button5 => mouse_button5,
			screen_shot => screen_shot,
			rec => rec,
		
			term_power_switch => term_power_switch,
			term_key => term_key,
			term_screen_shot => term_screen_shot,
			term_rec => term_rec,
			opt_screen => opt_screen,
			opt_vt102 => opt_vt102
		);

	-- generated media image components will be inserted here
	-- @MEDIA_IMAGE_COMPS@
end structural;
